import React, { useState, useCallback, useEffect, useRef } from 'react';
import {
  View,
  TouchableOpacity,
  Image,
  Modal,
  StyleSheet,
  Text,
  FlatList,
  Dimensions,
  Alert,
  ScrollView
} from 'react-native';
import {
  GiftedChat,
  Bubble,
  InputToolbar,
  Actions,
  IMessage,
  Send,
  BubbleProps,
  Composer
} from 'react-native-gifted-chat';
import { MaterialCommunityIcons } from '@expo/vector-icons';
import * as ImagePicker from 'expo-image-picker';
import { useActionSheet } from '@expo/react-native-action-sheet';
import {
  GestureHandlerRootView,
  LongPressGestureHandler,
  Swipeable
} from 'react-native-gesture-handler';
import { Header, PageWrapper } from 'src/components';
import { Colors } from 'src/theme';
import { useFocusEffect, useNavigation } from '@react-navigation/native';
import { Video } from 'expo-av';
import ChatMessageBox from '../Components/ChatMessageBox';
import ReplyMessageBar from '../Components/ReplyMessageBar';
import { useSharedValue, withTiming } from 'react-native-reanimated';
import { BlurView } from 'expo-blur';
import { useSafeAreaInsets } from 'react-native-safe-area-context';
import Clipboard from '@react-native-clipboard/clipboard';
import { trigger } from 'react-native-haptic-feedback';
import ReactModal from 'react-native-modal';
import { storage, StoreType } from 'src/storage';
import {
  usePostDeleteMessageMutation,
  usePostGetChatWithQuery,
  usePostMessagesReadMutation,
  usePostReactToMessageMutation,
  usePostSendMessageMutation
} from '@api/chat';
import { Message } from '../types';
import { API_HOST } from 'src/constants';
import { getFontSize } from 'src/utils';
import ReactionBar from '../Components/ReactionBar';
import OptionsMenu from '../Components/OptionsMenu';
import EmojiSelectorModal from '../Components/EmojiSelectorModal';

const options = {
  enableVibrateFallback: true,
  ignoreAndroidSystemSettings: false
};

const reactionEmojis = ['👍', '❤️', '😂', '😮', '😭'];

const ChatScreen = ({ route }: { route: any }) => {
  const { id, name, avatar }: { id: number; name: string; avatar: string | null } = route.params;
  const currentUserId = storage.get('uid', StoreType.STRING) as number;
  const token = storage.get('token', StoreType.STRING) as string;
  const insets = useSafeAreaInsets();
  const [messages, setMessages] = useState<IMessage[]>([]);
  const { showActionSheetWithOptions } = useActionSheet();
  const navigation = useNavigation();
  const { data: chatData, isFetching, refetch } = usePostGetChatWithQuery(token, id, -1, -1, true);
  const { mutateAsync: sendMessage } = usePostSendMessageMutation();

  const swipeableRowRef = useRef<Swipeable | null>(null);
  const messageContainerRef = useRef<FlatList<IMessage> | null>(null);
  const [selectedMedia, setSelectedMedia] = useState(null);

  const [replyMessage, setReplyMessage] = useState<IMessage | null>(null);

  const [selectedMessage, setSelectedMessage] = useState<IMessage | null>(null);
  const [emojiSelectorVisible, setEmojiSelectorVisible] = useState(false);
  const [messagePosition, setMessagePosition] = useState<{
    x: number;
    y: number;
    width: number;
    height: number;
    isMine: boolean;
  } | null>(null);

  const [isModalVisible, setIsModalVisible] = useState(false);
  const [unreadMessageIndex, setUnreadMessageIndex] = useState<number | null>(null);
  const { mutateAsync: markMessagesAsRead } = usePostMessagesReadMutation();
  const { mutateAsync: deleteMessage } = usePostDeleteMessageMutation();
  const { mutateAsync: reactToMessage } = usePostReactToMessageMutation();

  const [highlightedMessageId, setHighlightedMessageId] = useState<number | null>(null);

  const messageRefs = useRef<{ [key: string]: any }>({});
  const flatList = useRef<FlatList | null>(null);
  const scrollY = useSharedValue(0);

  const mapApiMessageToGiftedMessage = (message: Message): IMessage => {
    return {
      _id: message.id,
      text: message.text,
      createdAt: new Date(message.sent_datetime + 'Z'),
      user: {
        _id: message.sender,
        name: message.sender === id ? name : 'Me'
      },
      replyMessage:
        message.reply_to_id !== -1
          ? {
              text: message.reply_to.text,
              id: message.reply_to.id
            }
          : null,
      reactions: JSON.parse(message.reactions || '{}'),
      attachment: message.attachement !== -1 ? message.attachement : null,
      pending: message.status === 1,
      sent: message.status === 2,
      received: message.status === 3,
      deleted: message.status === 4
    };
  };

  useFocusEffect(
    useCallback(() => {
      if (chatData?.messages) {
        const mappedMessages = chatData.messages.map(mapApiMessageToGiftedMessage);

        const firstUnreadIndex = mappedMessages.findLastIndex(
          (msg) => !msg.received && !msg?.deleted && msg.user._id === id
        );
        if (firstUnreadIndex !== -1) {
          setUnreadMessageIndex(firstUnreadIndex);

          const unreadMarker: any = {
            _id: 'unreadMarker',
            text: 'Unread messages',
            system: true
          };

          mappedMessages.splice(firstUnreadIndex, 0, unreadMarker);
        }
        setMessages(mappedMessages);
      }
    }, [chatData])
  );

  const sentToServer = useRef<Set<number>>(new Set());

  const handleViewableItemsChanged = ({ viewableItems }: { viewableItems: any[] }) => {
    const newViewableUnreadMessages = viewableItems
      .filter(
        (item) =>
          !item.item.received &&
          !item.item.deleted &&
          !item.item.system &&
          item.item.user._id === id &&
          !sentToServer.current.has(item.item._id)
      )
      .map((item) => item.item._id);

    if (newViewableUnreadMessages.length > 0) {
      markMessagesAsRead(
        {
          token,
          from_user: id,
          messages_id: newViewableUnreadMessages
        },
        {
          onSuccess: (res) => {
            newViewableUnreadMessages.forEach((id) => sentToServer.current.add(id));
          }
        }
      );
    }
  };

  const renderSystemMessage = (props: any) => {
    if (props.currentMessage._id === 'unreadMarker') {
      return (
        <View style={styles.unreadMessagesContainer}>
          <Text style={styles.unreadMessagesText}>{props.currentMessage.text}</Text>
        </View>
      );
    }
    return null;
  };

  const clearReplyMessage = () => setReplyMessage(null);

  const handleLongPress = (message: IMessage, props) => {
    const messageRef = messageRefs.current[message._id];

    setSelectedMessage(props);
    trigger('impactMedium', options);

    const isMine = message.user._id === +currentUserId;

    if (messageRef) {
      messageRef.measureInWindow((x: number, y: number, width: number, height: number) => {
        const screenHeight = Dimensions.get('window').height;
        const spaceAbove = y - insets.top;
        const spaceBelow = screenHeight - (y + height) - insets.bottom * 2;

        let finalY = y;
        scrollY.value = 0;

        if (isNaN(y) || isNaN(height)) {
          console.error("Invalid measurement values for 'y' or 'height'", { y, height });
          return;
        }

        if (spaceBelow < 150) {
          const extraShift = 150 - spaceBelow;
          finalY -= extraShift;
        }

        if (spaceAbove < 50) {
          const extraShift = 50 - spaceAbove;
          finalY += extraShift;
        }

        if (spaceBelow < 150 || spaceAbove < 50) {
          const targetY = screenHeight / 2 - height / 2;
          scrollY.value = withTiming(finalY - finalY);
        }

        if (height > Dimensions.get('window').height - 200) {
          finalY = 100;
        }

        finalY = isNaN(finalY) ? 0 : finalY;

        setMessagePosition({ x, y: finalY, width, height, isMine });
        setIsModalVisible(true);
      });
    }
  };

  const openEmojiSelector = () => {
    setEmojiSelectorVisible(true);
    trigger('impactLight', options);
  };

  const closeEmojiSelector = () => {
    setEmojiSelectorVisible(false);
  };

  const handleReactionPress = (emoji: string, messageId: number) => {
    if (emoji === '+') {
      openEmojiSelector();
    } else {
      addReaction(messageId, emoji);
    }
  };

  const handleDeleteMessage = (messageId: number) => {
    deleteMessage(
      {
        message_id: messageId,
        conversation_with_user: id
      },
      {
        onSuccess: (res) => {
          setMessages((prevMessages) => prevMessages.filter((msg) => msg._id !== messageId));
        }
      }
    );
  };

  const handleOptionPress = (option: string) => {
    switch (option) {
      case 'reply':
        setReplyMessage(selectedMessage?.currentMessage);
        setIsModalVisible(false);
        break;
      case 'copy':
        Clipboard.setString(selectedMessage?.currentMessage?.text ?? '');
        setIsModalVisible(false);
        Alert.alert('copied');
        break;
      case 'delete':
        handleDeleteMessage(selectedMessage?.currentMessage?._id);
        setIsModalVisible(false);
        break;
      case 'pin':
        setIsModalVisible(false);
        Alert.alert(option);
        break;
      default:
        break;
    }
    closeEmojiSelector();
  };

  const renderSelectedMessage = () =>
    selectedMessage && (
      <View
        style={{
          maxHeight: '80%',
          width: messagePosition?.width,
          position: 'absolute',
          top: messagePosition?.y,
          left: messagePosition?.x
        }}
      >
        <ScrollView>
          <Bubble
            {...selectedMessage}
            wrapperStyle={{
              right: { backgroundColor: Colors.DARK_BLUE },
              left: { backgroundColor: Colors.FILL_LIGHT }
            }}
            textStyle={{
              right: { color: Colors.FILL_LIGHT },
              left: { color: Colors.DARK_BLUE }
            }}
            renderTicks={(message: IMessage) => {
              return message.user._id === +currentUserId && message.received ? (
                <View style={{ paddingRight: 8, bottom: 2 }}>
                  <MaterialCommunityIcons name="check-all" size={16} color={Colors.WHITE} />
                </View>
              ) : message.user._id === +currentUserId && message.sent ? (
                <View style={{ paddingRight: 8, bottom: 2 }}>
                  <MaterialCommunityIcons name="check" size={16} color={Colors.WHITE} />
                </View>
              ) : message.user._id === +currentUserId && message.pending ? (
                <View style={{ paddingRight: 8, bottom: 2 }}>
                  <MaterialCommunityIcons name="check" size={16} color={Colors.WHITE} />
                </View>
              ) : null;
            }}
            renderCustomView={renderReplyMessageView}
          />
        </ScrollView>
      </View>
    );

  const handleBackgroundPress = () => {
    setIsModalVisible(false);
    setSelectedMessage(null);
    closeEmojiSelector();
  };

  useEffect(() => {
    navigation?.getParent()?.setOptions({
      tabBarStyle: {
        display: 'none'
      }
    });
  }, [navigation]);

  useEffect(() => {
    const intervalId = setInterval(() => {
      refetch();
    }, 5000);

    return () => clearInterval(intervalId);
  }, [refetch]);

  const onSend = useCallback(
    (newMessages: IMessage[] = []) => {
      if (replyMessage) {
        newMessages[0].replyMessage = {
          text: replyMessage.text,
          id: replyMessage._id
        };
      }
      const message = { ...newMessages[0], pending: true };

      sendMessage(
        {
          to_uid: id,
          text: message.text,
          reply_to_id: replyMessage ? (replyMessage._id as number) : -1
        },
        {
          onSuccess: (res) => console.log('res', res),
          onError: (err) => console.log('err', err)
        }
      );

      setMessages((previousMessages) => GiftedChat.append(previousMessages, [message]));
      clearReplyMessage();
    },
    [replyMessage]
  );

  const openActionSheet = () => {
    const options = ['Open Camera', 'Select from gallery', 'Cancel'];
    const cancelButtonIndex = 2;

    showActionSheetWithOptions(
      {
        options,
        cancelButtonIndex
      },
      async (buttonIndex) => {
        if (buttonIndex === 0) {
          openCamera();
        } else if (buttonIndex === 1) {
          openGallery();
        }
      }
    );
  };

  const openCamera = async () => {
    const permissionResult = await ImagePicker.requestCameraPermissionsAsync();

    if (permissionResult.granted === false) {
      alert('Permission denied to access camera');
      return;
    }

    const result = await ImagePicker.launchCameraAsync({
      mediaTypes: ImagePicker.MediaTypeOptions.All,
      quality: 1,
      allowsEditing: true
    });

    if (!result.canceled) {
      const newMedia = {
        _id: Date.now().toString(),
        createdAt: new Date(),
        user: { _id: +currentUserId, name: 'Me' },
        image: result.assets[0].type === 'image' ? result.assets[0].uri : null,
        video: result.assets[0].type === 'video' ? result.assets[0].uri : null
      };
      setMessages((previousMessages) => GiftedChat.append(previousMessages, [newMedia as any]));
    }
  };

  const openGallery = async () => {
    const permissionResult = await ImagePicker.requestMediaLibraryPermissionsAsync();

    if (permissionResult.granted === false) {
      alert('Denied');
      return;
    }

    const result = await ImagePicker.launchImageLibraryAsync({
      mediaTypes: ImagePicker.MediaTypeOptions.All,
      allowsMultipleSelection: true,
      quality: 1
    });

    if (!result.canceled && result.assets) {
      const imageMessages = result.assets.map((asset) => ({
        _id: Date.now().toString() + asset.uri,
        createdAt: new Date(),
        user: { _id: +currentUserId, name: 'Me' },
        image: asset.type === 'image' ? asset.uri : null,
        video: asset.type === 'video' ? asset.uri : null
      }));

      setMessages((previousMessages) =>
        GiftedChat.append(previousMessages, imageMessages as IMessage[])
      );
    }
  };

  const renderMessageVideo = (props: any) => {
    const { currentMessage } = props;

    if (currentMessage.video) {
      return (
        <LongPressGestureHandler onHandlerStateChange={(event) => handleLongPress(currentMessage)}>
          <TouchableOpacity
            onPress={() => setSelectedMedia(currentMessage.video)}
            style={styles.mediaContainer}
          >
            <Video
              source={{ uri: currentMessage.video }}
              style={styles.chatMedia}
              useNativeControls
            />
          </TouchableOpacity>
        </LongPressGestureHandler>
      );
    }

    return null;
  };

  const addReaction = (messageId: number, reaction: string) => {
    const updatedMessages = messages.map((msg) => {
      if (msg._id === messageId) {
        const updatedReactions = [
          ...(Array.isArray(msg.reactions) ? msg.reactions : []),
          { datetime: new Date().toISOString(), reaction: reaction, uid: +currentUserId }
        ];

        return {
          ...msg,
          reactions: updatedReactions
        };
      }
      return msg;
    });

    setMessages(updatedMessages);

    reactToMessage(
      { message_id: messageId, reaction: reaction, conversation_with_user: id },
      {
        onSuccess: (res) => console.log('res', res),
        onError: (err) => console.log('err', err)
      }
    );

    setIsModalVisible(false);
  };

  const updateRowRef = useCallback(
    (ref: any) => {
      if (
        ref &&
        replyMessage &&
        ref.props.children.props.currentMessage?._id === replyMessage._id
      ) {
        swipeableRowRef.current = ref;
      }
    },
    [replyMessage]
  );

  const renderCustomView = (props: BubbleProps<IMessage>) => {
    if (!props.currentMessage) {
      return null;
    }
    const { currentMessage } = props;

    if (!currentMessage) {
      return null;
    }

    return (
      <View style={{ position: 'relative' }}>
        {renderReplyMessageView(props)}

        {currentMessage.reactions &&
          Array.isArray(currentMessage.reactions) &&
          currentMessage.reactions.length > 0 && (
            <View
              style={[
                styles.reactionsContainer,
                currentMessage.user._id === id ? { right: -15 } : { left: -15 }
              ]}
            >
              {Object.entries(
                currentMessage.reactions.reduce((acc, { reaction }) => {
                  acc[reaction] = (acc[reaction] || 0) + 1;
                  return acc;
                }, {})
              ).map(([emoji, count]) => (
                <Text key={emoji} style={{}}>
                  {emoji}
                  {(count as number) > 1 ? ` ${count}` : ''}
                </Text>
              ))}
            </View>
          )}
      </View>
    );
  };

  const renderReplyMessageView = (props: BubbleProps<IMessage>) => {
    if (!props.currentMessage) {
      return null;
    }
    const { currentMessage } = props;

    if (!currentMessage || !currentMessage?.replyMessage) {
      return null;
    }

    return (
      <TouchableOpacity
        style={styles.replyMessageContainer}
        onPress={() => {
          if (currentMessage?.replyMessage?.id) {
            scrollToMessage(currentMessage.replyMessage.id);
          }
        }}
      >
        <Text>{currentMessage.replyMessage.text}</Text>
        <View style={styles.replyMessageDivider} />
      </TouchableOpacity>
    );
  };

  const scrollToMessage = (messageId: number) => {
    const messageIndex = messages.findIndex((message) => message._id === messageId);

    if (messageIndex !== -1 && flatList.current) {
      flatList.current.scrollToIndex({
        index: messageIndex,
        animated: true,
        viewPosition: 0.5
      });

      setHighlightedMessageId(messageId);

      setTimeout(() => {
        setHighlightedMessageId(null);
      }, 1000);
    }
  };

  useEffect(() => {
    if (replyMessage && swipeableRowRef.current) {
      swipeableRowRef.current.close();
      swipeableRowRef.current = null;
    }
  }, [replyMessage]);

  const renderMessageImage = (props: any) => {
    const { currentMessage } = props;
    return (
      <TouchableOpacity
        onPress={() => setSelectedMedia(currentMessage.image)}
        style={styles.imageContainer}
      >
        <Image source={{ uri: currentMessage.image }} style={styles.chatImage} resizeMode="cover" />
      </TouchableOpacity>
    );
  };

  const renderBubble = (props: any) => {
    const { currentMessage } = props;

    if (currentMessage.deleted) {
      return (
        <View>
          <Bubble
            {...props}
            renderTime={() => null}
            currentMessage={{ ...props.currentMessage, text: 'This message was deleted' }}
            renderMessageText={() => (
              <View style={{ paddingHorizontal: 12, paddingVertical: 6 }}>
                <Text style={{ color: Colors.LIGHT_GRAY, fontStyle: 'italic', fontSize: 12 }}>
                  This message was deleted
                </Text>
              </View>
            )}
            wrapperStyle={{
              right: {
                backgroundColor: Colors.DARK_BLUE
              },
              left: {
                backgroundColor: Colors.FILL_LIGHT
              }
            }}
            textStyle={{
              left: {
                color: Colors.DARK_BLUE
              },
              right: {
                color: Colors.FILL_LIGHT
              }
            }}
          />
        </View>
      );
    }

    const isHighlighted = currentMessage._id === highlightedMessageId;
    const backgroundColor = isHighlighted
      ? Colors.ORANGE
      : currentMessage.user._id === +currentUserId
        ? Colors.DARK_BLUE
        : Colors.FILL_LIGHT;

    return (
      <View
        style={[
          currentMessage.reactions &&
          Array.isArray(currentMessage.reactions) &&
          currentMessage.reactions.length > 0
            ? { paddingBottom: 15 }
            : {}
        ]}
      >
        <View
          ref={(ref) => {
            if (ref && currentMessage) {
              messageRefs.current[currentMessage._id] = ref;
            }
          }}
          collapsable={false}
        >
          <Bubble
            {...props}
            wrapperStyle={{
              right: {
                backgroundColor: backgroundColor
              },
              left: {
                backgroundColor: backgroundColor
              }
            }}
            textStyle={{
              left: {
                color: Colors.DARK_BLUE
              },
              right: {
                color: Colors.FILL_LIGHT
              }
            }}
            onLongPress={() => handleLongPress(currentMessage, props)}
            renderTicks={(message: IMessage) => {
              return message.user._id === +currentUserId && message.received ? (
                <View style={{ paddingRight: 8, bottom: 2 }}>
                  <MaterialCommunityIcons name="check-all" size={16} color={Colors.WHITE} />
                </View>
              ) : message.user._id === +currentUserId && message.sent ? (
                <View style={{ paddingRight: 8, bottom: 2 }}>
                  <MaterialCommunityIcons name="check" size={16} color={Colors.WHITE} />
                </View>
              ) : message.user._id === +currentUserId && message.pending ? (
                <View style={{ paddingRight: 8, bottom: 2 }}>
                  <MaterialCommunityIcons name="check" size={16} color={Colors.LIGHT_GRAY} />
                </View>
              ) : null;
            }}
            renderCustomView={renderCustomView}
            isCustomViewBottom={true} // should be false to show reply on top
            // bottomContainerStyle={{
            //   right: {
            //     backgroundColor: 'red'
            //   },
            //   left: {
            //     backgroundColor: 'red'
            //   }
            // }}
          />
        </View>
      </View>
    );
  };

  const renderInputToolbar = (props: any) => (
    <InputToolbar
      {...props}
      renderActions={() => (
        <Actions
          icon={() => <MaterialCommunityIcons name="plus" size={28} color={Colors.DARK_BLUE} />}
          // onPressActionButton={openActionSheet}
        />
      )}
    />
  );

  return (
    <PageWrapper style={{ marginLeft: 0, marginRight: 0 }}>
      <View style={{ paddingHorizontal: '5%' }}>
        <Header
          label={name}
          rightElement={
            <Image
              source={{ uri: API_HOST + avatar }}
              style={{
                width: 30,
                height: 30,
                borderRadius: 15,
                borderWidth: 1,
                borderColor: Colors.FILL_LIGHT
              }}
            />
          }
        />
      </View>

      <GestureHandlerRootView style={styles.container}>
        <GiftedChat
          messages={messages}
          listViewProps={{
            ref: flatList,
            showsVerticalScrollIndicator: false,
            initialNumToRender: 30,
            onViewableItemsChanged: handleViewableItemsChanged,
            viewabilityConfig: { itemVisiblePercentThreshold: 50 },
            initialScrollIndex: unreadMessageIndex ?? 0,
            onScrollToIndexFailed: (info: any) => {
              const wait = new Promise((resolve) => setTimeout(resolve, 300));
              wait.then(() => {
                flatList.current?.scrollToIndex({
                  index: info.index,
                  animated: true,
                  viewPosition: 0.5
                });
              });
            }
          }}
          renderSystemMessage={renderSystemMessage}
          onSend={(newMessages: IMessage[]) => onSend(newMessages)}
          user={{ _id: +currentUserId, name: 'Me' }}
          renderBubble={renderBubble}
          renderMessageImage={renderMessageImage}
          renderInputToolbar={renderInputToolbar}
          messageContainerRef={messageContainerRef}
          renderSend={(props) => (
            <View
              style={{
                flexDirection: 'row',
                height: 44,
                alignItems: 'center',
                justifyContent: 'center',
                gap: 14,
                paddingHorizontal: 14
              }}
            >
              {props.text?.trim() && (
                <Send
                  {...props}
                  containerStyle={{
                    justifyContent: 'center'
                  }}
                >
                  <MaterialCommunityIcons name="send" size={28} color={Colors.DARK_BLUE} />
                </Send>
              )}
              {!props.text?.trim() && (
                <>
                  {/* <MaterialCommunityIcons
                    name="microphone-outline"
                    size={28}
                    color={Colors.DARK_BLUE}
                  />

                  <MaterialCommunityIcons
                    name="camera-outline"
                    size={28}
                    color={Colors.DARK_BLUE}
                  /> */}
                  <MaterialCommunityIcons name="send" size={28} color={Colors.LIGHT_GRAY} />
                </>
              )}
            </View>
          )}
          textInputProps={styles.composer}
          placeholder=""
          renderMessage={(props) => (
            <ChatMessageBox
              {...props}
              updateRowRef={updateRowRef}
              setReplyOnSwipeOpen={setReplyMessage}
            />
          )}
          renderChatFooter={() => (
            <ReplyMessageBar clearReply={clearReplyMessage} message={replyMessage} />
          )}
          renderMessageVideo={renderMessageVideo}
          renderAvatar={null}
          maxComposerHeight={100}
          renderComposer={(props) => <Composer {...props} />}
          isCustomViewBottom={true}
          keyboardShouldPersistTaps="handled"
          shouldUpdateMessage={(props, nextProps) =>
            highlightedMessageId === nextProps.currentMessage._id
          }
          // inverted={true}
          // isTyping={true}
        />

        <Modal visible={!!selectedMedia} transparent={true}>
          <View style={styles.modalContainer}>
            {selectedMedia && selectedMedia?.includes('.mp4') ? (
              <Video
                source={{ uri: selectedMedia }}
                style={styles.fullScreenMedia}
                useNativeControls
              />
            ) : (
              <Image source={{ uri: selectedMedia ?? '' }} style={styles.fullScreenMedia} />
            )}
            <TouchableOpacity onPress={() => setSelectedMedia(null)} style={styles.closeButton}>
              <MaterialCommunityIcons name="close" size={30} color="white" />
            </TouchableOpacity>
          </View>
        </Modal>

        <ReactModal
          isVisible={isModalVisible}
          onBackdropPress={handleBackgroundPress}
          style={styles.reactModalContainer}
          animationIn="fadeIn"
          animationOut="fadeOut"
          useNativeDriver
          backdropColor="transparent"
        >
          <BlurView
            intensity={80}
            style={styles.modalBackground}
            experimentalBlurMethod="dimezisBlurView"
          >
            <TouchableOpacity
              style={styles.modalBackground}
              activeOpacity={1}
              onPress={handleBackgroundPress}
            >
              <ReactionBar
                messagePosition={messagePosition}
                selectedMessage={selectedMessage}
                reactionEmojis={reactionEmojis}
                handleReactionPress={handleReactionPress}
                openEmojiSelector={openEmojiSelector}
              />
              {renderSelectedMessage()}
              <OptionsMenu
                selectedMessage={selectedMessage}
                handleOptionPress={handleOptionPress}
                messagePosition={messagePosition}
              />
              <EmojiSelectorModal
                visible={emojiSelectorVisible}
                selectedMessage={selectedMessage}
                addReaction={addReaction}
                closeEmojiSelector={closeEmojiSelector}
              />
            </TouchableOpacity>
          </BlurView>
        </ReactModal>
      </GestureHandlerRootView>
    </PageWrapper>
  );
};

const styles = StyleSheet.create({
  unreadMessagesContainer: {
    padding: 8,
    backgroundColor: Colors.FILL_LIGHT,
    alignItems: 'center'
  },
  unreadMessagesText: {
    color: Colors.DARK_BLUE,
    fontWeight: '700',
    fontSize: getFontSize(12)
  },
  modalBackground: {
    flex: 1
  },
  modalContent: {
    backgroundColor: 'transparent'
  },
  mediaContainer: {
    borderRadius: 10,
    overflow: 'hidden',
    margin: 5
  },
  chatMedia: {
    width: 200,
    height: 200,
    borderRadius: 10
  },
  fullScreenMedia: {
    width: '90%',
    height: '80%'
  },
  audioContainer: {
    flexDirection: 'row',
    alignItems: 'center',
    justifyContent: 'space-between',
    padding: 10,
    backgroundColor: '#f1f1f1',
    borderRadius: 10,
    margin: 5
  },
  progressBar: {
    flexDirection: 'row',
    justifyContent: 'center',
    alignItems: 'center'
  },
  bar: {
    width: 5,
    backgroundColor: 'gray',
    marginHorizontal: 1
  },
  replyMessageContainer: {
    padding: 8,
    paddingBottom: 0
  },
  replyMessageDivider: {
    borderBottomWidth: 1,
    borderBottomColor: 'lightgrey',
    paddingTop: 6
  },
  composer: {
    backgroundColor: Colors.FILL_LIGHT,
    borderRadius: 15,
    borderWidth: 1,
    borderColor: Colors.LIGHT_GRAY,
    paddingHorizontal: 10,
    fontSize: 16,
    marginVertical: 4
  },
  container: {
    flex: 1,
    backgroundColor: 'white'
  },
  imageContainer: {
    borderRadius: 10,
    overflow: 'hidden',
    margin: 5
  },
  chatImage: {
    width: 200,
    height: 200,
    borderRadius: 10
  },
  replyContainer: {
    backgroundColor: '#f1f1f1',
    padding: 5,
    marginBottom: 5,
    borderRadius: 5
  },
  replyText: {
    color: '#333',
    fontStyle: 'italic'
  },
  reactionsContainer: {
    height: 25,
    paddingHorizontal: 6,
    backgroundColor: Colors.FILL_LIGHT,
    bottom: -30,
    position: 'absolute',
    alignItems: 'center',
    justifyContent: 'center',
    flexDirection: 'row',
    gap: 6,
    borderRadius: 15,
    shadowColor: Colors.DARK_BLUE,
    shadowOpacity: 0.2,
    shadowOffset: { width: 0, height: 1 },
    shadowRadius: 2,
    elevation: 1
  },
  reactionText: {
    fontSize: 16,
    marginHorizontal: 5
  },
  reactionsBubble: {
    backgroundColor: '#fff',
    borderRadius: 20,
    padding: 5,
    flexDirection: 'row',
    shadowColor: '#000',
    shadowOpacity: 0.3,
    shadowOffset: { width: 0, height: 2 },
    shadowRadius: 5,
    elevation: 5,
    marginBottom: 10
  },
  replyInputContainer: {
    flexDirection: 'row',
    alignItems: 'center',
    backgroundColor: '#f1f1f1',
    paddingHorizontal: 10,
    paddingVertical: 5,
    borderRadius: 10
  },
  replyInputText: {
    flex: 1,
    color: 'gray'
  },
  modalContainer: {
    flex: 1,
    backgroundColor: 'rgba(0, 0, 0, 0.9)',
    justifyContent: 'center',
    alignItems: 'center'
  },
  reactModalContainer: {
    justifyContent: 'flex-end',
    margin: 0
  },
  fullScreenImage: {
    width: '90%',
    height: '80%'
  },
  closeButton: {
    position: 'absolute',
    top: 40,
    right: 20
  }
});

export default ChatScreen;
